#include "VideoSteuerung.h"
#include <DateiSystem.h>
#include <Text.h>
#include <Punkt.h>
#include <Bildschirm.h>
#include <MausEreignis.h>
#include <Bild.h>

// Inhalt der VideoSteuerung Klasse aus VideoSteuerung.h
// Konstruktor
VideoSteuerung::VideoSteuerung( UIInit &uiFactory )
    : ReferenceCounter()
{
    pos = uiFactory.initParam.bildschirm->getBackBufferSize() - Punkt( 285, 30 );
    verlassen = uiFactory.createKnopf( uiFactory.initParam );
    verlassen->setPosition( 0, 0 );
    verlassen->setSize( 100, 22 );
    verlassen->setText( "Verlassen" );
    verlassen->setToolTipText( "Aufzeichnung verlassen", uiFactory.initParam.bildschirm, uiFactory.initParam.schrift );
    langsamer = uiFactory.createKnopf( uiFactory.initParam );
    langsamer->setPosition( 105, 0 );
    langsamer->setSize( 22, 22 );
    langsamer->setStyle( Knopf::Style::Sichtbar | Knopf::Style::Erlaubt | Knopf::Style::HAlpha | Knopf::Style::HBild | Knopf::Style::Hintergrund | Knopf::Style::KlickBuffer | Knopf::Style::Rahmen );
    langsamer->setRahmenBreite( 1 );
    langsamer->setToolTipText( "Langsammer abspielen", uiFactory.initParam.bildschirm, uiFactory.initParam.schrift );
    stop = uiFactory.createKnopf( uiFactory.initParam );
    stop->setPosition( 130, 0 );
    stop->setSize( 22, 22 );
    stop->setStyle( Knopf::Style::Sichtbar | Knopf::Style::Erlaubt | Knopf::Style::HAlpha | Knopf::Style::HBild | Knopf::Style::Hintergrund | Knopf::Style::KlickBuffer | Knopf::Style::Rahmen );
    stop->setRahmenBreite( 1 );
    stop->setToolTipText( "Pause", uiFactory.initParam.bildschirm, uiFactory.initParam.schrift );
    schneller = uiFactory.createKnopf( uiFactory.initParam );
    schneller->setPosition( 155, 0 );
    schneller->setSize( 22, 22 );
    schneller->setStyle( Knopf::Style::Sichtbar | Knopf::Style::Erlaubt | Knopf::Style::HAlpha | Knopf::Style::HBild | Knopf::Style::Hintergrund | Knopf::Style::KlickBuffer | Knopf::Style::Rahmen );
    schneller->setRahmenBreite( 1 );
    schneller->setToolTipText( "Schneller abspielen", uiFactory.initParam.bildschirm, uiFactory.initParam.schrift );
    LTDBDatei *vBilder = new LTDBDatei();
    vBilder->setDatei( new Text( "data/client/bilder/video.ltdb" ) );
    vBilder->leseDaten( 0 );
    langsamer->setHintergrundBildZ( vBilder->laden( 0, new Text( "zurck.png" ) ) );
    stop->setHintergrundBildZ( vBilder->laden( 0, new Text( "pause.png" ) ) );
    schneller->setHintergrundBildZ( vBilder->laden( 0, new Text( "vor.png" ) ) );
    vBilder->release();
    zeit = uiFactory.createTextFeld( uiFactory.initParam );
    zeit->setPosition( 180, 0 );
    zeit->setSize( 100, 22 );
    zeit->addStyle( TextFeld::Style::Center );
    zeit->setText( "0:0:0" );
    spielZeit = 0;
    speed = 1;
    vorwrts = 1;
    zeitlupe = 0;
    beendet = 0;
    bJetzt = 0;
}

// Destruktor
VideoSteuerung::~VideoSteuerung()
{
    verlassen->release();
    langsamer->release();
    stop->release();
    schneller->release();
    zeit->release();
}

// Nicht constant
void VideoSteuerung::doPublicMausEreignis( MausEreignis &me )
{
    me.mx -= pos.x;
    me.my -= pos.y;
    bool vera = me.verarbeitet;
    verlassen->doPublicMausEreignis( me );
    if( !vera && me.verarbeitet )
    {
        if( me.id == ME_RLinks )
        {
            bJetzt = 1;
            beendet = 1;
        }
        vera = 1;
    }
    langsamer->doPublicMausEreignis( me );
    if( !vera && me.verarbeitet )
    {
        if( me.id == ME_RLinks )
        {
            if( !speed )
            {
                vorwrts = 0;
                speed = 1;
                stop->addStyle( Knopf::Style::Erlaubt );
                schneller->addStyle( Knopf::Style::Erlaubt );
            }
            else
            {
                speed += ( vorwrts && zeitlupe ) || ( !vorwrts && !zeitlupe );
                speed -= ( vorwrts && !zeitlupe ) || ( !vorwrts && zeitlupe );
                zeitlupe |= vorwrts && ( speed == 0 );
                zeitlupe &= vorwrts || ( speed != 0 );
                speed += speed == 0;
            }
        }
        vera = 1;
    }
    stop->doPublicMausEreignis( me );
    if( !vera && me.verarbeitet )
    {
        if( me.id == ME_RLinks )
        {
            speed = 0;
            zeitlupe = 0;
            stop->removeStyle( Knopf::Style::Erlaubt );
        }
        vera = 1;
    }
    schneller->doPublicMausEreignis( me );
    if( !vera && me.verarbeitet )
    {
        if( me.id == ME_RLinks )
        {
            if( !speed )
            {
                vorwrts = 1;
                speed = 1;
                stop->addStyle( Knopf::Style::Erlaubt );
                langsamer->addStyle( Knopf::Style::Erlaubt );
            }
            else
            {
                speed += ( vorwrts && !zeitlupe ) || ( !vorwrts && zeitlupe );
                speed -= ( vorwrts && zeitlupe ) || ( !vorwrts && !zeitlupe );
                zeitlupe |= !vorwrts && ( speed == 0 );
                zeitlupe &= !vorwrts || ( speed != 0 );
                speed += speed == 0;
            }
        }
        vera = 1;
    }
    me.mx += pos.x;
    me.my += pos.y;
}

bool VideoSteuerung::tick( double zeit )
{
    bool ret = langsamer->tick( zeit );
    ret |= verlassen->tick( zeit );
    ret |= stop->tick( zeit );
    ret |= schneller->tick( zeit );
    spielZeit += zeit;
    if( (int)spielZeit != (int)( spielZeit - zeit ) )
    {
        Text z;
        z = (int)spielZeit / 60 / 60;
        z += ":";
        z += (int)spielZeit / 60 % 60;
        z += ":";
        z += (int)spielZeit % 60;
        this->zeit->setText( z );
    }
    ret |= this->zeit->tick( zeit );
    return ret;
}

void VideoSteuerung::render( Bild &zRObj )
{
    if( !zRObj.setDrawOptions( pos.x, pos.y, 300, 30 ) )
        return;
    verlassen->render( zRObj );
    langsamer->render( zRObj );
    stop->render( zRObj );
    schneller->render( zRObj );
    zeit->render( zRObj );
    zRObj.releaseDrawOptions();
}

void VideoSteuerung::pause( bool ende )
{
    speed = 0;
    zeitlupe = 0;
    stop->removeStyle( Knopf::Style::Erlaubt );
    if( ende )
        schneller->removeStyle( Knopf::Style::Erlaubt );
    else
        langsamer->removeStyle( Knopf::Style::Erlaubt );
}

bool VideoSteuerung::istBeendet( bool jetzt )
{
    if( jetzt )
    {
        if( bJetzt )
        {
            bJetzt = 0;
            return 1;
        }
        return 0;
    }
    return beendet;
}

// constant
int VideoSteuerung::getSpeed() const
{
    return speed + zeitlupe;
}

bool VideoSteuerung::istVorwrts() const
{
    return vorwrts;
}

bool VideoSteuerung::istZeitlupe() const
{
    return zeitlupe;
}